@@ -33,10 +33,13 @@
 #include "bosonweapon.h"
 #include "bosonstatistics.h"
 #include "bosonprofiling.h"
+#include "bosoncanvasstatistics.h"
 #include "bodebug.h"
 #include "boson.h"
+#include "bosonpropertyxml.h"
 
 #include <klocale.h>
+#include <kgame/kgamepropertyhandler.h>
 
 #include <qpointarray.h>
 #include <qdatastream.h>
@@ -46,6 +49,23 @@
 
 #include "bosoncanvas.moc"
 
+ItemType ItemType::typeForUnit(unsigned long int unitType)
+{
+ return ItemType(unitType);
+}
+ItemType ItemType::typeForExplosion()
+{
+ return ItemType(BosonShot::Explosion, 0, 0);
+}
+ItemType ItemType::typeForFragment()
+{
+ return ItemType(BosonShot::Fragment, 0, 0);
+}
+ItemType ItemType::typeForShot(unsigned long int shotType, unsigned long int unitType, unsigned long int weaponPropertyId)
+{
+ return ItemType(shotType, unitType, weaponPropertyId);
+}
+
 
 class BosonCanvas::BosonCanvasPrivate
 {
@@ -53,9 +73,13 @@ public:
 	BosonCanvasPrivate()
 		: mAllItems(BoItemList(0, false))
 	{
+		mStatistics = 0;
 		mMap = 0;
+
+		mProperties = 0;
 	}
-	
+	BosonCanvasStatistics* mStatistics;
+
 	QPtrList<Unit> mDestroyedUnits;
 
 	BosonMap* mMap; // just a pointer - no memory allocated
@@ -63,8 +87,6 @@ public:
 	QPtrList<BosonItem> mAnimList; // see BosonCanvas::slotAdvance()
 
 	BoItemList mAllItems;
-	QMap<int, unsigned int> mItemCount;
-
 	QPtrList<BosonParticleSystem> mParticles;
 
 	// by default ALL items are in "work" == -1. if an item changes its work
@@ -73,8 +95,16 @@ public:
 	QMap<int, QPtrList<BosonItem> > mWork2AdvanceList;
 	QPtrList<BosonItem> mChangeAdvanceList; // work has been changed - request to change advance list
 
-	// For debugging only
-	QMap<int, int> mWorkCounts; // How many units are doing what work
+	KGamePropertyHandler* mProperties;
+
+	// AB: we _really_ need ulong here. for _very_ long games (maybe more
+	// than a few days playing without break and a lot of shots) we might
+	// even exceed that.
+	// at the moment we won't be able to have so long games and won't have
+	// _that_ big battles. in the future (if we maybe add even more items)
+	// we might use unsigned long long for this.
+	// but for now we are safe.
+	KGameProperty<unsigned long int> mNextItemId;
 };
 
 BosonCanvas::BosonCanvas(QObject* parent)
@@ -90,17 +120,28 @@ void BosonCanvas::init()
  d->mParticles.setAutoDelete(true);
  mAdvanceFunctionLocked = false;
  mCollisions = new BosonCollisions();
+ d->mStatistics = new BosonCanvasStatistics(this);
+ d->mProperties = new KGamePropertyHandler(this);
+ d->mNextItemId.registerData(IdNextItemId, d->mProperties,
+		KGamePropertyBase::PolicyLocal, "NextItemId");
+ d->mNextItemId.setLocal(0);
 }
 
 BosonCanvas::~BosonCanvas()
 {
 boDebug()<< k_funcinfo << endl;
  quitGame();
+ delete d->mStatistics;
  delete mCollisions;
  delete d;
 boDebug()<< k_funcinfo <<"done"<< endl;
 }
 
+BosonCanvasStatistics* BosonCanvas::canvasStatistics() const
+{
+ return d->mStatistics;
+}
+
 void BosonCanvas::quitGame()
 {
  deleteDestroyed(); // already called before
@@ -111,6 +152,7 @@ void BosonCanvas::quitGame()
 	(*it).clear();
  }
  d->mChangeAdvanceList.clear();
+ d->mNextItemId = 0;
 }
 
 void BosonCanvas::deleteDestroyed()
@@ -135,17 +177,6 @@ Cell* BosonCanvas::cells() const
  return d->mMap->cells();
 }
 
-void BosonCanvas::slotAddUnit(Unit* unit, int x, int y)
-{
- if (!unit) {
-	boError() << k_funcinfo << "NULL unit!" << endl;
-	return;
- }
-
- unit->move(x, y, unit->z());
-// unit->show();
-}
-
 void BosonCanvas::slotAdvance(unsigned int advanceCount, bool advanceFlag)
 {
 #define USE_ADVANCE_LISTS 1
@@ -154,7 +185,7 @@ void BosonCanvas::slotAdvance(unsigned int advanceCount, bool advanceFlag)
  QPtrListIterator<BosonItem> animIt(d->mAnimList);
  lockAdvanceFunction();
  boProfiling->advanceFunction(true);
- resetWorkCounts();
+ d->mStatistics->resetWorkCounts();
 
 #if USE_ADVANCE_LISTS
  // first we need to call *all* BosonItem::advance() functions.
@@ -172,7 +203,7 @@ void BosonCanvas::slotAdvance(unsigned int advanceCount, bool advanceFlag)
 		id = 0;
 		work = -1;
 	}
-	d->mWorkCounts[work]++;
+	d->mStatistics->increaseWorkCount(work);
 #if DO_ITEM_PROFILING
 	boProfiling->advanceItemStart(s->rtti(), id, work);
 	boProfiling->advanceItem(true);
@@ -217,9 +248,7 @@ void BosonCanvas::slotAdvance(unsigned int advanceCount, bool advanceFlag)
 			}
 			break;
 		case (int)UnitBase::WorkDestroyed:
-			if (advanceCount % 20 != 0) {
-				skip = true;
-			}
+			skip = false;
 			break;
 		case (int)UnitBase::WorkFollow:
 			if (advanceCount % 5 != 0) {
@@ -283,6 +312,21 @@ void BosonCanvas::slotAdvance(unsigned int advanceCount, bool advanceFlag)
 	}
  }
 
+ // now we need to make sure that the correct advance function will be called in
+ // the next advance call.
+ animIt.toFirst();
+ if (advanceFlag) {
+	for (; animIt.current(); ++animIt) {
+		BosonItem* i = animIt.current();
+		i->syncAdvanceFunction();
+	}
+ } else {
+	for (; animIt.current(); ++animIt) {
+		BosonItem* i = animIt.current();
+		i->syncAdvanceFunction2();
+	}
+ }
+
  // we completed iterating through the advance lists. now we might have to
  // change the list for some items.
  QPtrListIterator<BosonItem> changeIt(d->mChangeAdvanceList);
@@ -398,10 +442,6 @@ void BosonCanvas::slotAdvance(unsigned int advanceCount, bool advanceFlag)
  boProfiling->advanceFunction(false);
  unlockAdvanceFunction();
 
- boProfiling->advanceDeleteUnusedShots(true);
- deleteUnusedShots(); //AB: do we *have* to do this in every advance call???
- boProfiling->advanceDeleteUnusedShots(false);
-
  boProfiling->advanceParticles(true);
  updateParticleSystems(0.05);  // With default game speed, delay between advance messages is 1.0 / 20 = 0.05 sec
  boProfiling->advanceParticles(false);
@@ -428,6 +468,10 @@ void BosonCanvas::slotAdvance(unsigned int advanceCount, bool advanceFlag)
 	}
 
 	deleteUnits(&deleteList);
+
+	boProfiling->advanceDeleteUnusedShots(true);
+	deleteUnusedShots();
+	boProfiling->advanceDeleteUnusedShots(false);
  }
  boProfiling->advanceMaximalAdvanceCount(false);
  boProfiling->advance(false, advanceCount);
@@ -543,20 +587,9 @@ void BosonCanvas::updateSight(Unit* unit, float , float)
  }
 }
 
-void BosonCanvas::newShot(BosonShot* shot)
+void BosonCanvas::newShot(BosonShot*)
 {
  boDebug(350) << k_funcinfo << endl;
-
- if (!shot->isActive()) {
-	shotHit(shot);
-
-	// AB: is this a good idea? maybe prefer deleteUnusedShots() instead!
-	// (note that deleteUnusedShots() depends on the item still being in 
-	// the animation list!!)
-	// it won't hurt too much here, cause setAnimated(true) got never called
-	// for a new shot that is inactive. but its ugly anyway.
-	delete shot;
- }
 }
 
 void BosonCanvas::shotHit(BosonShot* s)
@@ -573,14 +606,16 @@ void BosonCanvas::shotHit(BosonShot* s)
 		++it;
 	}
  }
- // Add hit particle systems
- addParticleSystems(s->properties()->newHitParticleSystems(BoVector3(s->x(), s->y(), s->z())));
+ if (s->properties()) {
+	// Add hit particle systems
+	addParticleSystems(s->properties()->newHitParticleSystems(BoVector3(s->x(), s->y(), s->z())));
 
- // Play hit sound
- s->properties()->playSound(SoundWeaponHit);
+	// Play hit sound
+	s->properties()->playSound(SoundWeaponHit);
+ }
 
- explosion(BoVector3(s->x(), s->y(), s->z()), s->properties()->damage(), s->properties()->damageRange(),
-		s->properties()->fullDamageRange(), s->owner());
+ explosion(BoVector3(s->x(), s->y(), s->z()), s->damage(), s->damageRange(),
+		s->fullDamageRange(), s->owner());
 }
 
 void BosonCanvas::explosion(const BoVector3& pos, long int damage, float range, float fullrange, Player* owner)
@@ -710,6 +745,7 @@ void BosonCanvas::destroyUnit(Unit* unit)
 	// This stops everything
 	unit->stopAttacking();
 
+	// Stop particle emitting for all systems this unit has
 	if (unit->particleSystems() && unit->particleSystems()->count() > 0) {
 		QPtrListIterator<BosonParticleSystem> it(*(unit->particleSystems()));
 		for (; it.current(); ++it) {
@@ -725,13 +761,35 @@ void BosonCanvas::destroyUnit(Unit* unit)
 	// Pos is center of unit
 	BoVector3 pos(unit->x() + unit->width() / 2, unit->y() + unit->height() / 2, unit->z());
 	//pos += unit->unitProperties()->hitPoint();
+	// Add destroyed particle systems
 	addParticleSystems(unit->unitProperties()->newDestroyedParticleSystems(pos[0], pos[1], pos[2]));
-	if(unit->unitProperties()->explodingDamage() > 0) {
+	// Make explosion if needed
+	const UnitProperties* prop = unit->unitProperties();
+	if (prop->explodingDamage() > 0) {
+		BosonShotExplosion* e = (BosonShotExplosion*)createNewItem(RTTI::Shot, unit->owner(), ItemType(BosonShot::Explosion, 0, 0), pos);
 		// Do we want ability to set fullDamageRange here?
-		explosion(pos, unit->unitProperties()->explodingDamage(), unit->unitProperties()->explodingDamageRange(), 0, unit->owner());
+		if (e) {
+			// AB: pos parameter is redundant due to createNewItem()
+			// change
+			e->activate(pos, prop->explodingDamage(), prop->explodingDamageRange(), 0.0f, 10);
+		}
 	}
+	// Add explosion fragments
+	for (unsigned int i = 0; i < unit->unitProperties()->explodingFragmentCount(); i++) {
+		BosonShotFragment* f = (BosonShotFragment*)createNewItem(RTTI::Shot, unit->owner(), ItemType(BosonShot::Fragment, 0, 0), pos);
+		if (f) {
+			// AB: pos parameter is redundant due to createNewItem()
+			// change
+			f->activate(pos, unit->unitProperties());
+		}
+	}
+	// Hide unit if wreckage should be removed immediately
+	if (unit->unitProperties()->removeWreckageImmediately()) {
+		unit->setVisible(false);
+	}
+	// Check if owner is out of game
 	if (owner->checkOutOfGame()) {
-		killPlayer(owner);
+		boGame->killPlayer(owner);
 	}
  }
 }
@@ -782,7 +840,7 @@ unsigned int BosonCanvas::mapHeight() const
  return map() ? map()->height() : 0;
 }
 
-float* BosonCanvas::heightMap() const
+const float* BosonCanvas::heightMap() const
 {
  return map() ? map()->heightMap() : 0;
 }
@@ -802,15 +860,27 @@ float BosonCanvas::heightAtCorner(int x, int y) const
  return map()->heightAtCorner(x, y);
 }
 
-void BosonCanvas::killPlayer(Player* player)
+float BosonCanvas::heightAtPoint(float x, float y) const
 {
- while (player->allUnits()->count() > 0) {
-	destroyUnit(player->allUnits()->first());
- }
- player->setMinerals(0);
- player->setOil(0);
- boDebug() << k_funcinfo << "player " << player->id() << " is out of game" << endl;
- emit signalOutOfGame(player);
+ // Coordinates of the cell (x; y) is on
+ int cellX = (int)(x / BO_TILE_SIZE);
+ int cellY = (int)(y / BO_TILE_SIZE);
+
+ // Will be used as factors for blending
+ float x2 = (x / BO_TILE_SIZE) - cellX;
+ float y2 = (y / BO_TILE_SIZE) - cellY;
+
+ // These are heights of the corners of the cell (x; y) is on
+ float h1, h2, h3, h4;
+
+ h1 = heightAtCorner(cellX, cellY);
+ h2 = heightAtCorner(cellX + 1, cellY);
+ h3 = heightAtCorner(cellX, cellY + 1);
+ h4 = heightAtCorner(cellX + 1, cellY + 1);
+
+ // Blend all corners together and return the result
+ // FIXME: this can probably be written _much_ more understandably and maybe faster
+ return ((h1 * (1 - x2) + (h2 * x2)) * (1 - y2)) + ((h3 * (1 - x2) + (h4 * x2)) * y2);
 }
 
 void BosonCanvas::removeFromCells(BosonItem* item)
@@ -891,14 +961,6 @@ bool BosonCanvas::canPlaceUnitAtCell(const UnitProperties* prop, const QPoint& p
  return false;
 }
 
-BosonTiles* BosonCanvas::tileSet() const
-{
- if (!d->mMap) {
-	return 0;
- }
- return d->mMap->tileSet();
-}
-
 BoItemList* BosonCanvas::allItems() const
 {
  return &d->mAllItems;
@@ -920,33 +982,12 @@ void BosonCanvas::removeItem(BosonItem* item)
  emit signalRemovedItem(item);
 }
 
-unsigned int BosonCanvas::itemCount(int rtti) const
+unsigned int BosonCanvas::particleSystemsCount() const
 {
- return d->mItemCount[rtti];
+ return particleSystems()->count();
 }
 
-void BosonCanvas::updateItemCount()
-{
- d->mItemCount.clear();
- BoItemList::Iterator it = d->mAllItems.begin();
- for (; it != d->mAllItems.end(); ++it) {
-	int rtti = (*it)->rtti();
-	if (RTTI::isUnit(rtti)) {
-		rtti = RTTI::UnitStart;
-	}
-	if (!d->mItemCount.contains(rtti)) {
-		d->mItemCount.insert(rtti, 0);
-	}
-	d->mItemCount[rtti] += 1;
- }
-}
-
-int BosonCanvas::particleSystemsCount() const
-{
- return d->mParticles.count();
-}
-
-QPtrList<BosonParticleSystem>* BosonCanvas::particleSystems()
+QPtrList<BosonParticleSystem>* BosonCanvas::particleSystems() const
 {
  return &(d->mParticles);
 }
@@ -982,7 +1023,6 @@ void BosonCanvas::deleteUnusedShots()
 	if (RTTI::isShot(i->rtti())) {
 		BosonShot* shot = (BosonShot*)i;
 		if (!shot->isActive()) {
-			shotHit(shot);
 			delete i;
 		}
 	}
@@ -1007,55 +1047,6 @@ unsigned int BosonCanvas::animationsCount() const
  return d->mAnimList.count();
 }
 
-bool BosonCanvas::save(QDataStream& stream)
-{
- boDebug() << k_funcinfo << endl;
- // Save shots
- // Count first. This is bad because we have to iterate through list twice
- Q_UINT32 shotscount = 0;
- for (BosonItem* i = d->mAnimList.first(); i; i = d->mAnimList.next()) {
-	if (RTTI::isShot(i->rtti())) {
-		shotscount++;
-	}
- }
- stream << shotscount;
- boDebug() << k_funcinfo << "Saving " << shotscount << " shots" << endl;
-
- BosonShot* s;
- for (BosonItem* i = d->mAnimList.first(); i; i = d->mAnimList.next()) {
-	if (RTTI::isShot(i->rtti())) {
-		boDebug() << k_funcinfo << "Saving shot" << endl;
-		s = (BosonShot*)i;
-		stream << (Q_UINT32)s->owner()->id();
-		stream << (Q_UINT32)s->properties()->unitProperties()->typeId();
-		stream << (Q_UINT32)s->properties()->id();
-		s->save(stream);
-	}
- }
- return true;
-}
-
-bool BosonCanvas::load(QDataStream& stream)
-{
- boDebug() << k_funcinfo << endl;
- Q_UINT32 shotscount;
- stream >> shotscount;
- boDebug() << k_funcinfo << "Loading " << shotscount << " shots" << endl;
-
- BosonShot* s;
- Player* p;
- Q_UINT32 playerid, unitpropid, propid;
- for (unsigned int i = 0; i < shotscount; i++) {
-	boDebug() << k_funcinfo << "Loading shot" << endl;
-	stream >> playerid >> unitpropid >> propid;
-	p = (Player*)boGame->findPlayer(playerid);
-	s = new BosonShot(p->speciesTheme()->unitProperties(unitpropid)->weaponProperties(propid),
-			p, this);
-	s->load(stream);
- }
- return true;
-}
-
 bool BosonCanvas::loadFromXML(const QDomElement& root)
 {
  if (root.isNull()) {
@@ -1063,106 +1054,271 @@ bool BosonCanvas::loadFromXML(const QDomElement& root)
 	return false;
  }
 
- // Load shots
- QDomElement shots = root.namedItem(QString::fromLatin1("Shots")).toElement();
- if (shots.isNull()) {
-	boWarning(260) << k_funcinfo << "no shots " << endl;
-	return true;
- }
- QDomNodeList list = shots.elementsByTagName(QString::fromLatin1("Shot"));
- if (list.count() == 0) {
-	// It's ok to have no shots
-	return true;
- }
 
+ QDomNodeList list = root.elementsByTagName(QString::fromLatin1("Items"));
+ unsigned int itemCount = 0;
  for (unsigned int i = 0; i < list.count(); i++) {
-	QDomElement e = list.item(i).toElement();
-	if (e.isNull()) {
-		boError(260) << k_funcinfo << i << " is not an element" << endl;
-		return false;
-	}
-	if (!e.hasAttribute(QString::fromLatin1("Owner"))) {
-		boError(260) << k_funcinfo << "missing attribute: Owner for Shot " << i << endl;
-		continue;
-	}
-	if (!e.hasAttribute(QString::fromLatin1("UnitType"))) {
-		boError(260) << k_funcinfo << "missing attribute: UnitType for Shot " << i << endl;
-		continue;
-	}
-	if (!e.hasAttribute(QString::fromLatin1("WeaponType"))) {
-		boError(260) << k_funcinfo << "missing attribute: WeaponType for Shot " << i << endl;
+	QDomElement items = list.item(i).toElement();
+	if (items.isNull()) {
+		boError(260) << k_funcinfo << "Items tag is not an element" << endl;
 		continue;
 	}
 	bool ok = false;
-	unsigned long int type, weaponid, ownerid;
-	ownerid = e.attribute(QString::fromLatin1("Owner")).toULong(&ok);
+	unsigned int ownerId = items.attribute(QString::fromLatin1("OwnerId")).toUInt(&ok);
 	if (!ok) {
-		boError(260) << k_funcinfo << "Invalid Owner number for Shot " << i << endl;
+		boError(260) << k_funcinfo << "OwnerId of Items Tag " << i << " is not a valid number" << endl;
 		continue;
 	}
-	type = e.attribute(QString::fromLatin1("UnitType")).toULong(&ok);
-	if (!ok) {
-		boError(260) << k_funcinfo << "Invalid UnitType number for Shot " << i << endl;
-		continue;
-	}
-	weaponid = e.attribute(QString::fromLatin1("WeaponType")).toULong(&ok);
-	if (!ok) {
-		boError(260) << k_funcinfo << "Invalid WeaponType number for Shot " << i << endl;
-		continue;
-	}
-
-	Player* owner = (Player*)(boGame->findPlayer(ownerid));
+	Player* owner = (Player*)boGame->findPlayer(ownerId);
 	if (!owner) {
-		boError() << k_funcinfo << "No player with id " << ownerid << endl;
-	}
-	SpeciesTheme* theme = owner->speciesTheme();
-	if (!theme) {
-		boError() << k_funcinfo << "No theme for player " << ownerid << endl;
+		// AB: this is totally valid. less players in game, than in the
+		// file.
 		continue;
 	}
-	const UnitProperties* prop = theme->unitProperties(type);
-	if (!prop) {
-		boError() << "Unknown unitType " << type << endl;
-		return 0;
-	}
-	const BosonWeaponProperties* weapon = prop->weaponProperties(weaponid);
-	if (!weapon) {
-		boError() << "Unknown weaponType " << weaponid << " for unitType " << type << endl;
-		return 0;
-	}
 
-	BosonShot* shot = new BosonShot(weapon, owner, this);
-
-	// Call shot's loading methods
-	if (!shot->loadFromXML(e)) {
-		boWarning(260) << k_funcinfo << "Could not load shot " << i << " correctly" << endl;
-		delete shot;
-		continue;
+	// FIXME: the unit target must be loaded after _all_ units are loaded
+	QDomNodeList itemList = items.elementsByTagName(QString::fromLatin1("Item"));
+	for (unsigned int j = 0; j < itemList.count(); j++) {
+		QDomElement item = itemList.item(j).toElement();
+		if (item.isNull()) {
+			continue;
+		}
+		if (!loadItemFromXML(item, owner)) {
+			boError(260) << k_funcinfo << "Item " << j << " was not loaded correctly" << endl;
+			continue;
+		}
+		itemCount++;
 	}
  }
+ QDomElement handler = root.namedItem("DataHandler").toElement();
+ if (handler.isNull()) {
+	boError() << k_funcinfo << "DataHandler not found" << endl;
+	return false;
+ }
+ BosonPropertyXML propertyXML;
+ if (!propertyXML.loadFromXML(handler, d->mProperties)) {
+	boError() << k_funcinfo << "unable to load the datahandler" << endl;
+	return false;
+ }
+ boDebug(260) << k_funcinfo << "loaded " << itemCount << " items" << endl;
  return true;
 }
 
+bool BosonCanvas::loadItemFromXML(const QDomElement& item, Player* owner)
+{
+ if (item.isNull()) {
+	return false;
+ }
+ if (!owner) {
+	return false;
+ }
+ bool ok = false;
+ int rtti = item.attribute(QString::fromLatin1("Rtti")).toInt(&ok);
+ if (!ok) {
+	boError(260) << k_funcinfo << "Rtti attribute of Item is not a valid number" << endl;
+	return false;
+ }
+
+ unsigned long int type = 0;
+ unsigned long int group = 0;
+ unsigned long int groupType = 0;
+
+ if (!item.hasAttribute(QString::fromLatin1("Type"))) {
+	// check for deprecated attributes
+	if (!item.hasAttribute(QString::fromLatin1("UnitType"))) {
+		boError(260) << k_funcinfo << "missing attribute: Type for Item tag" << endl;
+		return false;
+	} else {
+		type = item.attribute(QString::fromLatin1("UnitType")).toULong(&ok);
+	}
+ } else {
+	type = item.attribute(QString::fromLatin1("Type")).toULong(&ok);
+ }
+ if (!ok) {
+	boError(260) << k_funcinfo << "Invalid Type number for Item tag" << endl;
+	return false;
+ }
+
+ if (item.hasAttribute(QString::fromLatin1("Group"))) {
+	group = item.attribute(QString::fromLatin1("Group")).toULong(&ok);
+ } else {
+	// check for deprecated attributes.
+	if (item.hasAttribute(QString::fromLatin1("UnitType")) && item.hasAttribute(QString::fromLatin1("Type"))) {
+		// old Shot tags used "Type" for the type and "UnitType" for the
+		// "Group" attribute.
+		group = item.attribute(QString::fromLatin1("UnitType")).toULong(&ok);
+	} else {
+		// "Group" attribute is not present.
+		// this is totally valid! items don't have to provide a Group
+		// attribute.
+		ok = true;
+	}
+ }
+ if (!ok) {
+	boError(260) << k_funcinfo << "Invalid Group number for Item tag" << endl;
+	return false;
+ }
+
+ if (item.hasAttribute(QString::fromLatin1("GroupType"))) {
+	groupType = item.attribute(QString::fromLatin1("GroupType")).toULong(&ok);
+ } else {
+	// check for deprecated attributes.
+	if (item.hasAttribute(QString::fromLatin1("WeaponType"))) {
+		groupType = item.attribute(QString::fromLatin1("WeaponType")).toULong(&ok);
+	}
+ }
+ if (!ok) {
+	boError(260) << k_funcinfo << "Invalid GroupType number for Item tag" << endl;
+	return false;
+ }
+
+ BoVector3 pos;
+ pos.setX(item.attribute("x").toFloat(&ok));
+ if (!ok) {
+	boError() << k_funcinfo << "x attribute for Item tag missing or invalid" << endl;
+	return false;
+ }
+ pos.setY(item.attribute("y").toFloat(&ok));
+ if (!ok) {
+	boError() << k_funcinfo << "y attribute for Item tag missing or invalid" << endl;
+	return false;
+ }
+ pos.setZ(item.attribute("z").toFloat(&ok));
+ if (!ok) {
+	// missing z is ok, but not recommended.
+	pos.setZ(0.0f);
+ }
+
+ if (RTTI::isUnit(rtti)) {
+	if (!item.hasAttribute(QString::fromLatin1("Id"))) {
+		boError(260) << k_funcinfo << "missing attribute: Id for Item tag" << endl;
+		return false;
+	}
+	if (!item.hasAttribute(QString::fromLatin1("DataHandlerId"))) {
+		boError(260) << k_funcinfo << "missing attribute: DataHandlerId for Item tag" << endl;
+		return false;
+	}
+	unsigned long int id = 0;
+	int dataHandlerId = -1;
+
+	// AB: "0" indicates that we want boson to assign an Id. the tag must
+	// always be present.
+	id = item.attribute(QString::fromLatin1("Id")).toULong(&ok);
+	if (!ok) {
+		boError(260) << k_funcinfo << "Invalid Id number for Item tag" << endl;
+		return false;
+	}
+	if (item.hasAttribute(QString::fromLatin1("DataHandlerId"))) {
+		dataHandlerId = item.attribute(QString::fromLatin1("DataHandlerId")).toInt(&ok);
+		if (!ok) {
+			boError(260) << k_funcinfo << "Invalid DataHandlerId number for Item tag" << endl;
+			return false;
+		}
+	}
+	if (id == 0) {
+		id = nextItemId();
+	}
+
+	// FIXME: I think we should move addUnit() to bosoncanvas.
+	//
+	// AB: TODO: createNewUnit() - which includes owner->addUnit()
+	// AB: maybe drop create_New_Item completely and move all it does to
+	// createItem().
+	// (i.e. the owner->addUnit() and the theme->loadNewUnit() call. also a
+	// few additional exceptions (editor, flying unit)).
+	Unit* u = (Unit*)createItem(RTTI::UnitStart + type, owner, ItemType(type), pos, id);
+
+	// Set additional properties
+	owner->addUnit(u, dataHandlerId);
+
+	if (u->speciesTheme()) {
+		u->speciesTheme()->loadNewUnit(u);
+	}
+
+	// Call unit's loading methods
+	if (!u->loadFromXML(item)) {
+		boWarning(260) << k_funcinfo << "Could not load unit " << id << " correctly" << endl;
+		// no need to return false
+		// also it is dangerous now, as we already called addUnit()!
+	}
+
+	return true;
+ } else if (RTTI::isShot(rtti)) {
+	BosonShot* s = (BosonShot*)createNewItem(RTTI::Shot, owner, ItemType(type, group, groupType), pos);
+	if (!s) {
+		boError() << k_funcinfo << "Invalid shot - type=" << type << " group=" << group << " groupType=" << groupType << endl;
+		return false;
+	}
+	if (!s->loadFromXML(item)) {
+		boWarning(260) << k_funcinfo << "Could not load shot correctly" << endl;
+		delete s;
+		return false;
+	}
+	ok = true;
+	return true;
+ } else {
+	boError(260) << k_funcinfo << "unknown Rtti " << rtti << endl;
+	return false;
+ }
+ return false;
+}
+
 bool BosonCanvas::saveAsXML(QDomElement& root)
 {
  boDebug() << k_funcinfo << endl;
 
- // Save shots
  QDomDocument doc = root.ownerDocument();
- QDomElement shots = doc.createElement(QString::fromLatin1("Shots"));
- BosonShot* s;
+ QMap<unsigned int, QDomElement> owner2Items;
+ for (KPlayer* p = boGame->playerList()->first(); p; p = boGame->playerList()->next()) {
+	QDomElement items = doc.createElement(QString::fromLatin1("Items"));
+	items.setAttribute(QString::fromLatin1("OwnerId"), p->id());
+	root.appendChild(items);
+	owner2Items.insert(p->id(), items);
+ }
+
  for (BosonItem* i = d->mAnimList.first(); i; i = d->mAnimList.next()) {
+	QDomElement items;
 	if (RTTI::isShot(i->rtti())) {
-		s = (BosonShot*)i;
-		QDomElement shot = doc.createElement(QString::fromLatin1("Shot"));
-		if (!s->saveAsXML(shot)) {
-			boError() << k_funcinfo << "Could not save shot " << s << endl;
+		BosonShot* s = (BosonShot*)i;
+		if (!s->owner()) {
+			BO_NULL_ERROR(s->owner());
 			continue;
 		}
-		shots.appendChild(shot);
+		unsigned int id = s->owner()->id();
+		items = owner2Items[id];
+	} else if (RTTI::isUnit(i->rtti())) {
+		Unit* u = (Unit*)i;
+		if (!u->owner()) {
+			BO_NULL_ERROR(u->owner());
+			continue;
+		}
+		unsigned int id = u->owner()->id();
+		items = owner2Items[id];
 	}
+	if (items.isNull()) {
+		boError() << k_funcinfo << "no Items element found" << endl;
+		continue;
+	}
+	QDomElement item = doc.createElement(QString::fromLatin1("Item"));
+	if (RTTI::isShot(i->rtti())) {
+		if (!((BosonShot*)i)->isActive()) {
+			continue;
+		}
+	}
+	if (!i->saveAsXML(item)) {
+		boError() << k_funcinfo << "Could not save item " << i << endl;
+		continue;
+	}
+	items.appendChild(item);
+ }
+
+ BosonPropertyXML propertyXML;
+ QDomElement handler = doc.createElement(QString::fromLatin1("DataHandler"));
+ root.appendChild(handler);
+ if (!propertyXML.saveAsXML(handler, d->mProperties)) {
+	boError() << k_funcinfo << "unable to save the datahandler" << endl;
+	return false;
  }
- root.appendChild(shots);
  return true;
 }
 
@@ -1193,24 +1349,6 @@ void BosonCanvas::removeFromAdvanceLists(BosonItem* item)
  }
 }
 
-void BosonCanvas::resetWorkCounts()
-{
- d->mWorkCounts[-1] = 0;
- d->mWorkCounts[(int)UnitBase::WorkNone] = 0;
- d->mWorkCounts[(int)UnitBase::WorkMove] = 0;
- d->mWorkCounts[(int)UnitBase::WorkAttack] = 0;
- d->mWorkCounts[(int)UnitBase::WorkConstructed] = 0;
- d->mWorkCounts[(int)UnitBase::WorkDestroyed] = 0;
- d->mWorkCounts[(int)UnitBase::WorkFollow] = 0;
- d->mWorkCounts[(int)UnitBase::WorkPlugin] = 0;
- d->mWorkCounts[(int)UnitBase::WorkTurn] = 0;
-}
-
-QMap<int, int>* BosonCanvas::workCounts() const
-{
- return &d->mWorkCounts;
-}
-
 bool BosonCanvas::onCanvas(const BoVector3& pos) const
 {
  return onCanvas((int)pos.x(), (int)pos.y());
@@ -1245,3 +1383,173 @@ void BosonCanvas::deleteUnits(QPtrList<Unit>* units)
  }
 }
 
+BosonItem* BosonCanvas::createNewItem(int rtti, Player* owner, const ItemType& type, const BoVector3& pos)
+{
+ BosonItem* item = createItem(rtti, owner, type, pos, nextItemId());
+ if (!item) {
+	return 0;
+ }
+ if (RTTI::isUnit(item->rtti())) {
+	Unit* unit = (Unit*)item;
+	if (unit->owner() != owner) {
+		boError() << k_funcinfo << "unexpected owner for new unit" << endl;
+		return item;
+	}
+	owner->addUnit(unit);
+	SpeciesTheme* theme = owner->speciesTheme();
+	if (!theme) {
+		boError() << k_funcinfo << "NULL speciesTheme" << endl;
+		return item;
+	}
+	theme->loadNewUnit(unit);
+	unit->setAnimationMode(UnitAnimationIdle);
+	if (unit->isFlying()) {
+		// AB: we have currently not decided how to treat flying units,
+		// so we just place them at a height of 2.0 on construction.
+		// note that on loading units this may break the positions and
+		// when the height of the ground is at 2.0, we don't recognize
+		// that either.
+		unit->move(unit->x(), unit->y(), 2.0f);
+	}
+
+
+	if (!boGame->gameMode()) {
+		// editor won't display the construction but always completed
+		// facilities. otherwise it's hard to recognize where they were actually
+		// placed
+		if (unit->glConstructionSteps() > 0) {
+			unit->setGLConstructionStep(unit->glConstructionSteps());
+		}
+		unit->setAnimationMode(UnitAnimationIdle);
+	}
+ }
+
+
+ return item;
+}
+
+BosonItem* BosonCanvas::createItem(int rtti, Player* owner, const ItemType& type, const BoVector3& pos, unsigned long int id)
+{
+ BosonItem* item = 0;
+ if (!onCanvas(pos)) {
+	boError() << k_funcinfo << pos.debugString() << " is not on the canvas" << endl;
+	return 0;
+ }
+ if (id == 0) {
+	boError() << k_funcinfo << "id==0 is invalid." << endl;
+	return 0;
+ }
+ if (RTTI::isUnit(rtti)) {
+	item = (BosonItem*)createUnit(owner, type.mType);
+ } else if (RTTI::isShot(rtti)) {
+	item = (BosonItem*)createShot(owner, type.mType, type.mGroup, type.mGroupType);
+ }
+ if (item) {
+	item->setId(id);
+	item->move(pos.x(), pos.y(), pos.z());
+	emit signalItemAdded(item);
+ }
+ return item;
+}
+
+Unit* BosonCanvas::createUnit(Player* owner, unsigned long int unitType)
+{
+ BO_CHECK_NULL_RET0(owner);
+ SpeciesTheme* theme = owner->speciesTheme();
+ BO_CHECK_NULL_RET0(theme); // BAAAAD - will crash
+
+ const UnitProperties* prop = theme->unitProperties(unitType);
+ if (!prop) {
+	boError() << k_funcinfo << "Unknown unitType " << unitType << endl;
+	return 0;
+ }
+
+ Unit* unit = 0;
+ if (prop->isMobile()) {
+	unit = new MobileUnit(prop, owner, this);
+ } else if (prop->isFacility()) {
+	unit = new Facility(prop, owner, this);
+ } else { // should be impossible
+	boError() << k_funcinfo << "invalid unit type " << unitType << endl;
+	return 0;
+ }
+ theme->loadNewUnit(unit);
+ return unit;
+}
+
+BosonShot* BosonCanvas::createShot(Player* owner, unsigned long int shotType, unsigned long int unitType, unsigned long int weaponPropertyId)
+{
+ BO_CHECK_NULL_RET0(owner);
+ BosonShot* s = 0;
+ switch (shotType) {
+	case BosonShot::Bullet:
+	{
+		SpeciesTheme* t = owner->speciesTheme();
+		BO_CHECK_NULL_RET0(t);
+		const UnitProperties* unitProperties = t->unitProperties(unitType);
+		BO_CHECK_NULL_RET0(unitProperties);
+		const BosonWeaponProperties* prop = unitProperties->weaponProperties(weaponPropertyId);
+		BO_CHECK_NULL_RET0(prop);
+		s = (BosonShot*)new BosonShotBullet(owner, this, prop);
+		break;
+	}
+	case BosonShot::Missile:
+	{
+		SpeciesTheme* t = owner->speciesTheme();
+		BO_CHECK_NULL_RET0(t);
+		const UnitProperties* unitProperties = t->unitProperties(unitType);
+		BO_CHECK_NULL_RET0(unitProperties);
+		const BosonWeaponProperties* prop = unitProperties->weaponProperties(weaponPropertyId);
+		BO_CHECK_NULL_RET0(prop);
+		s = (BosonShot*)new BosonShotMissile(owner, this, prop);
+		break;
+	}
+	case BosonShot::Explosion:
+		s = (BosonShot*)new BosonShotExplosion(owner, this);
+		break;
+	case BosonShot::Mine:
+	{
+		SpeciesTheme* t = owner->speciesTheme();
+		BO_CHECK_NULL_RET0(t);
+		const UnitProperties* unitProperties = t->unitProperties(unitType);
+		BO_CHECK_NULL_RET0(unitProperties);
+		const BosonWeaponProperties* prop = unitProperties->weaponProperties(weaponPropertyId);
+		BO_CHECK_NULL_RET0(prop);
+		s = (BosonShot*)new BosonShotMine(owner, this, prop);
+		break;
+	}
+	case BosonShot::Bomb:
+	{
+		SpeciesTheme* t = owner->speciesTheme();
+		BO_CHECK_NULL_RET0(t);
+		const UnitProperties* unitProperties = t->unitProperties(unitType);
+		BO_CHECK_NULL_RET0(unitProperties);
+		const BosonWeaponProperties* prop = unitProperties->weaponProperties(weaponPropertyId);
+		BO_CHECK_NULL_RET0(prop);
+		s = (BosonShot*)new BosonShotBomb(owner, this, prop);
+		break;
+	}
+	case BosonShot::Fragment:
+	{
+		SpeciesTheme* t = owner->speciesTheme();
+		BO_CHECK_NULL_RET0(t);
+		BosonModel* model = t->objectModel("fragment");
+		BO_CHECK_NULL_RET0(model);
+		s = (BosonShot*)new BosonShotFragment(owner, this, model);
+		break;
+	}
+	default:
+		boError() << k_funcinfo << "Invalid type: " << shotType << endl;
+		s = 0;
+		break;
+ }
+ return s;
+}
+
+unsigned long int BosonCanvas::nextItemId()
+{
+ // note that per definition 0 is an invalid item ID!
+ d->mNextItemId = d->mNextItemId + 1;
+ return d->mNextItemId;
+}
+
