@@ -23,7 +23,7 @@
 
 #include <GL/gl.h>
 
-#include <qptrvector.h>
+#include <qglobal.h>
 
 class BosonCanvas;
 class BosonCollisions;
@@ -34,24 +34,113 @@ class BoFrame;
 class QRect;
 class Cell;
 class BosonParticleSystem;
+class BosonItemPropertyHandler;
+
+class KGamePropertyHandler;
+class KGamePropertyBase;
 class QColor;
 template<class T> class QPtrList;
+template<class T> class QPtrVector;
+template<class T1, class T2> class QMap;
+class QDomElement;
+class QDataStream;
+class QString;
+
+/**
+ * This is the base class of @ref BosonItem. It exists only to logically
+ * separate the code that provides access to @ref KGamePropertyHandler and
+ * friends from all other code.
+ *
+ * You will most probably want to look at @ref BosonItem instead.
+ *
+ * @author Andreas Beckermann <b_mann@gmx.de>
+ **/
+class BosonItemProperties
+{
+public:
+	BosonItemProperties();
+	virtual ~BosonItemProperties();
+
+	virtual int rtti() const = 0;
+
+	/**
+	 * Initialize static members
+	 **/
+	static void initStatic();
+
+	/**
+	 * Add a property ID to the list of properties. This must be done before
+	 * calling @ref registerData.
+	 * @param id A <em>unique</em> property ID - you must ensure that one
+	 * unit never uses two identical ids.
+	 * @param name The name of the property. Will be used in the debug
+	 * dialog as well as e.g. in the scenario files. This name must be
+	 * unique as well!
+	 **/
+	static void addPropertyId(int id, const QString& name);
+
+	/**
+	 * @return The id of the specified property name. Or -1 if not found.
+	 * See @ref addPropertyId.
+	 **/
+	static int propertyId(const QString& name);
+
+	/**
+	 * @return A name for the specified property id or QString::null if not
+	 * found. See also @ref addPropertyId
+	 **/
+	static QString propertyName(int id);
+
+	/**
+	 * Shortcut for
+	 * <pre>
+	 * prop->registerData(id, dataHandler(), KGamePropertyBase::PolicyLocal,
+	 * propertyName(id));
+	 * </pre>
+	 *
+	 * Note that you must call @ref addPropertyId before you are able to use
+	 * registerData!
+	 * @param prop The @ref KGamePropertyBase to be registered
+	 * @param id The PropertyId for the @ref KGamePropertyBase. This must be
+	 * unique for every property, i.e. a unit must never have two identical
+	 * property ids.
+	 * @param local If TRUE use @ref KGamePropertyBase::PolicyLocal,
+	 * otherwise @ref KGamePropertyBase::PolicyClean. Don't use FALSE here
+	 * unless you know what you're doing!
+	 **/
+	void registerData(KGamePropertyBase* prop, int id, bool local = true);
+
+	/**
+	 * @return The @ref KGamePropertyHandler for all the properties of this
+	 * item. It is in fact a @ref BosonItemPropertyHandler, you can cast to
+	 * it safely.
+	 **/
+	KGamePropertyHandler* dataHandler() const;
+
+private:
+	BosonItemPropertyHandler* mProperties;
+	static QMap<int, QString>* mPropertyMap;
+
+};
 
 
 /**
  * This is the base class for all visual items, i.e. a OpenGL objects in boson.
- * Subclasses can be e.g. a missile object, units or mines.
+ * Subclasses can be e.g. a missile objects, units or mines.
  *
- * Every BosonItem must have a model assigned, which will be used to render the
- * object onto the screen. You also have to use a <em>unique</em> @ref rtti for
- * every item, i.e. a missile must have a different rtti than the units.
+ * Every BosonItem should have a model assigned, which will be used to render
+ * the object onto the screen. If you don't want the item to be rendered or if
+ * you don't have a model, you can use @ref setVisible to prevent item from
+ * being rendered
+ * You also have to use a <em>unique</em> @ref rtti for every item, i.e. a
+ * missile must have a different rtti than the units.
  *
  * Note that only <em>canvas</em> coordinates are stored in this class,
  * <em>not</em> OpenGL coordinates!
  * @short Base class for all visual OpenGL objects in boson.
  * @author Andreas Beckermann <b_mann@gmx.de>
  **/
-class BosonItem
+class BosonItem : public BosonItemProperties
 {
 public:
 	/**
@@ -71,6 +160,18 @@ public:
 	 **/
 	virtual int rtti() const = 0;
 
+	/**
+	 * Set a unique Id for this item. The Id <em>must</em> be unique for
+	 * <em>all</em> items in the game. Otherwise the results are undefined.
+	 **/
+	void setId(unsigned long int id) { mId = id; }
+
+	/**
+	 * @return An id that identifies this item uniquely. There are never 2
+	 * different items with the same Id.
+	 **/
+	inline unsigned long int id() const { return mId; }
+
 	/**
 	 * @return The x-coordinate of the left edge of the object.Note that
 	 * this has <em>nothing</em> to do with the OpenGL coordinates. 
@@ -87,9 +188,7 @@ public:
 	inline float y() const { return mY; }
 
 	/**
-	 * @return The z-position of the unit. Currently 0.0 for all units,
-	 * except flying units. Might be changed to the height of the ground one
-	 * day
+	 * @return The z-position of the item.
 	 **/
 	inline float z() const { return mZ; }
 
@@ -97,12 +196,19 @@ public:
 	 * @param width Width in cells * BO_TILE_SIZE. FIXME: should be changed to cell value (float!!)
 	 * @param height Height in cells * BO_TILE_SIZE. FIXME: should be changed to cell value (float!!)
 	 **/
-	void setSize(int width, int height);
+	void setSize(int width, int height, float depth);
 
 	// note: for GLunit all frames must have the same width/height!
 	// different depth is ok!
 	inline int width() const { return mWidth; }
 	inline int height() const { return mHeight; }
+	/**
+	 * @return item's height in z-direction.
+	 * This does not depend on any OpenGL stuff (model, frame etc) and should be
+	 * same on all clients, so it can be used for collision detection and
+	 * pathfinding
+	 **/
+	inline float depth() const { return mDepth; }
 
 	inline float leftEdge() const { return x(); }
 	inline float topEdge() const { return y(); }
@@ -159,10 +265,9 @@ public:
 	/**
 	 * Render the item. This assumes the modelview matrix was already
 	 * translated and rotated to the correct position.
-	 * @param teamColor The color that should get used for teamcolored areas
-	 * of the mode. Use NULL if you don't want a team-colored object.
+	 * @param lod See @ref BoFrame::renderFrame
 	 **/
-	void renderItem();
+	void renderItem(unsigned int lod = 0);
 
 	inline GLuint displayList() const
 	{
@@ -252,6 +357,48 @@ public:
 		mZVelocity = vz;
 	}
 
+	/**
+	 * @return Current speed of this item
+	 **/
+	inline float speed() const { return mCurrentSpeed; }
+	inline void setSpeed(float s) { mCurrentSpeed = s; }
+	/**
+	 * @return Maximum speed this item may have
+	 **/
+	inline float maxSpeed() const { return mMaxSpeed; }
+	inline void setMaxSpeed(float maxspeed) { mMaxSpeed = maxspeed; }
+	/**
+	 * Raises speed by @ref accelerationSpeed unless @ref currentSpeed is
+	 * @ref maxSpeed
+	 **/
+	inline void accelerate() { mCurrentSpeed = QMIN(maxSpeed(), speed() + accelerationSpeed()); }
+	/**
+	 * Lowers speed by @ref decelerationSpeed unless @ref currentSpeed is 0
+	 **/
+	inline void decelerate() { mCurrentSpeed = QMAX(0, speed() - decelerationSpeed()); }
+	/**
+	 * @return How fast this unit accelerates.
+	 * Acceleration speed shows how much speed of unit changes per advance call.
+	 **/
+	inline float accelerationSpeed() const { return mAccelerationSpeed; }
+	inline void setAccelerationSpeed(float s) { mAccelerationSpeed = s; }
+	/**
+	 * @return How fast this unit decelerates.
+	 * Deceleration speed shows how much speed of unit changes per advance call.
+	 **/
+	inline float decelerationSpeed() const { return mDecelerationSpeed; }
+	inline void setDecelerationSpeed(float s) { mDecelerationSpeed = s; }
+	/**
+	 * @return How much this unit moves before stopping completely
+	 * This is distance that item will move before it completely stops when it
+	 * starts deceleration now and continues it until stopping
+	 **/
+	inline float decelerationDistance() const { return (mCurrentSpeed / mDecelerationSpeed) / 2 * mCurrentSpeed; }
+
+
+	inline void setVisible(bool v) { mIsVisible = v; }
+	inline bool isVisible() const { return mIsVisible; }
+
 	/**
 	 * Set the animation mode. Only possible if the construction of the unit
 	 * is completed (i.e. the construction step is greater or equal to @ref
@@ -283,7 +430,9 @@ public:
 	 **/
 	inline virtual void advance(unsigned int )
 	{
-		animate();
+		if (mCurrentAnimation) {
+			animate();
+		}
 	}
 
 	/**
@@ -296,6 +445,21 @@ public:
 	 **/
 	inline virtual void advanceFunction2(unsigned int /*advanceCount*/) { }
 
+	/**
+	 * Used to synchronize the advance function for the next advance call.
+	 * See @ref Unit::syncAdvanceFunction.
+	 *
+	 * DO NOT CALL THIS unless you REALLY know what youre doing! Call it
+	 * from @ref BosonCanvas::slotAdvance ONLY!
+	 **/
+	inline virtual void syncAdvanceFunction() { }
+
+	/**
+	 * Used to synchronize the advance function for the next advance call.
+	 * See @ref Unit::syncAdvanceFunction.
+	 **/
+	inline virtual void syncAdvanceFunction2() { }
+
 	/**
 	 * @return The factor you need to multiply BO_GL_CELL_SIZE with to
 	 * achieve the depth (height in z-direction) of the unit. Note that this
@@ -368,6 +532,9 @@ public:
 	 **/
 	virtual void unselect();
 
+	virtual bool saveAsXML(QDomElement&);
+	virtual bool loadFromXML(const QDomElement&);
+
 protected:
 	/**
 	 * @return The team color this item should get rendered with. This
@@ -438,16 +605,23 @@ private:
 	BosonAnimation* mCurrentAnimation;
 	// FIXME: use KGameProperty here. We can do so, since we don't use
 	// QCanvasSprite anymore.
+	unsigned long int mId;
 	float mX;
 	float mY;
 	float mZ;
 	int mWidth;
 	int mHeight;
+	float mDepth;
 
 	float mXVelocity;
 	float mYVelocity;
 	float mZVelocity;
 
+	float mCurrentSpeed;
+	float mMaxSpeed;
+	float mAccelerationSpeed;
+	float mDecelerationSpeed;
+
 // OpenGL values. should not be used for pathfinding and so on. Most stoff
 // shouldn't be stored in save() either
 	float mRotation;
@@ -473,6 +647,7 @@ private:
 
 	QPtrVector<Cell>* mCells;
 	bool mCellsDirty;
+	bool mIsVisible;
 };
 
 #endif
